﻿using RimWorld;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Verse;
using Verse.AI;

namespace MyMod.Buildings
{
    public class Bulding_MyTestBuilding : Building
    {
        public CompPowerTrader CompPowerTrader;

        public bool HasPower
        {
            get
            {
                if(CompPowerTrader != null && CompPowerTrader.PowerOn)
                {
                    return !Map.gameConditionManager.ConditionIsActive(GameConditionDefOf.SolarFlare);
                }

                return false;
            }
        }

        public Thing ContainedThing;

        public bool Complete;

        private int ticks => 15000;
        private int ticker = 0;

        public MyComponent MyComponent
        {
            get
            {
                if (myComponent == null)
                {
                    myComponent = Current.Game.GetComponent<MyComponent>();
                }

                return myComponent;
            }
        }
        private MyComponent myComponent;

        public override void SpawnSetup(Map map, bool respawningAfterLoad)
        {
            base.SpawnSetup(map, respawningAfterLoad);

            CompPowerTrader = GetComp<CompPowerTrader>();

            MyComponent.Num++;

            Current.Game.World.GetComponent<MyWorldComponent>().Num2++;
        }

        public override void Tick()
        {
            base.Tick();

            if (!HasPower)
                return;

            if(ContainedThing != null && !Complete)
            {
                ticker--;

                if(ticker <= 0)
                {
                    ThingComplete();
                }
            }
        }

        private void ThingComplete()
        {
            Complete = true;

            if(ContainedThing != null)
            {
                ContainedThing.stackCount *= 2;
            }
        }

        public void LoadItem(Thing item)
        {
            ContainedThing = item;

            item.DeSpawn();

            ticker = ticks;
        }

        public void GetItem()
        {
            GenDrop.TryDropSpawn(ContainedThing, Position, Map, ThingPlaceMode.Near, out Thing result);

            ContainedThing = null;
        }

        public override string GetInspectString()
        {
            StringBuilder builder = new StringBuilder();
            builder.Append($"Bulding_MyTestBuilding_InspectString".Translate(ticker.TicksToDays().ToString("f2")));
            if(Complete)
                builder.Append($"Bulding_MyTestBuilding_InspectStringStatus".Translate());

            return builder.ToString();
        }

        public override IEnumerable<FloatMenuOption> GetFloatMenuOptions(Pawn selPawn)
        {
            if (ContainedThing != null)
            {
                yield return new FloatMenuOption("Bulding_MyTestBuilding_TakeJob_UnloadItem".Translate(), delegate
                {
                    Job job = new Job(JobDefOfLocal.GetItemFromBuilding, this);
                    job.playerForced = true;

                    selPawn.jobs.TryTakeOrderedJob(job);
                });
            }
            else
            {
                yield return new FloatMenuOption("Bulding_MyTestBuilding_TakeJob_LoadItem".Translate(), delegate
                {
                    List<FloatMenuOption> options = new List<FloatMenuOption>();

                    foreach (var thing in GetThings())
                    {
                        options.Add(new FloatMenuOption(thing.Label, delegate
                        {
                            Job job = new Job(JobDefOfLocal.CarryItemToBuilding, this, thing);
                            job.count = thing.stackCount;
                            job.playerForced = true;

                            selPawn.jobs.TryTakeOrderedJob(job);
                        }));
                    }

                    if (options.Count == 0)
                        return;

                    Find.WindowStack.Add(new FloatMenu(options));
                });
            }
        }

        private IEnumerable<Thing> GetThings()
        {
            return Map.listerThings.ThingsInGroup(ThingRequestGroup.Drug);
        }

        public override void ExposeData()
        {
            base.ExposeData();

            Scribe_References.Look(ref ContainedThing, "ContainedThing");
            Scribe_Values.Look(ref ticker, "ticker");
        }
    }
}
