﻿using RimWorld;
using RimWorld.Planet;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Verse;

namespace MyMod.WorldObjects
{
    public class CaravanArrivalAction_EnterToMap : CaravanArrivalAction
    {
        private MapParent mapParent;

        public override string Label => "Do Enter";

        public override string ReportString => "entering";

        public CaravanArrivalAction_EnterToMap() : base()
        {

        }

        public CaravanArrivalAction_EnterToMap(MapParent mapParent)
        {
            this.mapParent = mapParent;
        }

        public override void Arrived(Caravan caravan)
        {
            LongEventHandler.QueueLongEvent(delegate
            {
                DoEnter(caravan);
            }, "GeneratingMapForNewEncounter", doAsynchronously: false, null);
        }

        private void DoEnter(Caravan caravan)
        {
            bool hasMap = mapParent.HasMap;
            Map map = GetOrGenerateMapUtility.GetOrGenerateMap(mapParent.Tile, new IntVec3(80, 1, 80), null);
            if (hasMap)
            {
                //TODO
            }
            else
            {
                //TODO
            }
            CaravanEnterMapUtility.Enter(caravan, map, CaravanEnterMode.Edge);

        }

        public override FloatMenuAcceptanceReport StillValid(Caravan caravan, int destinationTile)
        {
            FloatMenuAcceptanceReport floatMenuAcceptanceReport = base.StillValid(caravan, destinationTile);
            if (!floatMenuAcceptanceReport)
            {
                return floatMenuAcceptanceReport;
            }
            if (mapParent != null && mapParent.Tile != destinationTile)
            {
                return false;
            }

            return CanVisit(caravan, mapParent);
        }

        public override void ExposeData()
        {
            base.ExposeData();
            Scribe_References.Look(ref mapParent, "mapParent");
        }

        public static FloatMenuAcceptanceReport CanVisit(Caravan caravan, MapParent mapParent)
        {
            if (mapParent == null || !mapParent.Spawned)
            {
                return false;
            }

            if(caravan.pawns.Count < 2)
            {
                return FloatMenuAcceptanceReport.WithFailReason("Need > 2 peoples");
            }

            return true;
        }
    }
}
